@@ -2,6 +2,10 @@
 
 /* expense conduit, based on read-expense */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
 #include <gnome.h>
 
@@ -10,6 +14,7 @@
 #include <pi-dlp.h>
 #include <pi-version.h>
 
+#include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <utime.h>
@@ -84,6 +89,7 @@ static void
 save_configuration(ConduitCfg *c) 
 {
 	gchar *prefix;
+        char buf[20];
 
 	g_assert(c!=NULL);
 
@@ -93,10 +99,10 @@ save_configuration(ConduitCfg *c)
 	gnome_config_set_string("dir", c->dir);
 	gnome_config_set_string("date_format", c->dateFormat);
 	gnome_config_set_int("output_format", c->outputFormat);
-	g_snprintf(prefix,255,"0%o", c->dirMode);
-	gnome_config_set_string("dir mode", prefix);
-	g_snprintf(prefix,255,"0%o", c->fileMode);
-	gnome_config_set_string("file mode", prefix);
+	g_snprintf(buf,sizeof(buf),"0%o", c->dirMode);
+	gnome_config_set_string("dir mode", buf);
+	g_snprintf(buf,sizeof(buf),"0%o", c->fileMode);
+	gnome_config_set_string("file mode", buf);
 
 	gnome_config_pop_prefix();
 
@@ -178,8 +184,8 @@ static gchar *category_path(int category, GnomePilotConduit *c)
 	static gchar filename[FILENAME_MAX];
 	gchar buf[64];
 	
-	if(category==16) 
-		strcpy(buf,"Archived");
+	if(category < 0 || category >= 16)
+		strcpy(buf,"BogusCategory");
 	else
 		protect_name(buf,GET_CONDUIT_DATA(c)->ai.category.name[category]);
   
@@ -236,17 +242,25 @@ package, Copyright (c) 1997, Kenneth Albanowski
 static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
 {
         int dbHandle;
+#ifdef PILOT_LINK_0_12
+        pi_buffer_t *pi_buf;
+#else
         guchar buffer[0xffff];
+#endif
 
         struct ExpenseAppInfo *tai;
         struct ExpensePref *tp;
 
         int i;
         int ret;
-        int filehandle[17];
+        const int numhandles = 16; /* number of categories. See pi-appinfo.h  */
+        int filehandle[numhandles];
         int result = 0;
 
-        if(GET_CONDUIT_CFG(c)->dir==NULL) {
+        for (i = 0; i < numhandles; i++)
+                filehandle[i] = -1;
+
+        if(GET_CONDUIT_CFG(c)->dir==NULL || strlen(GET_CONDUIT_CFG(c)->dir)==0) {
 		g_warning(_("No dir specified. Please run expense conduit capplet first."));
 		gnome_pilot_conduit_send_error(c,
                                                _("No dir specified. Please run expense conduit capplet first."));
@@ -262,7 +276,12 @@ static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
                 return -1;
         }
     
+#ifdef PILOT_LINK_0_12
+        pi_buf = pi_buffer_new (0xffff);     
+        unpack_ExpensePref(tp, pi_buf->data, 0xffff);        
+#else
         unpack_ExpensePref(tp, buffer, 0xffff);
+#endif
 
 #ifdef EC_DEBUG
         fprintf(stderr, "Orig prefs, %d bytes:\n", ret);
@@ -278,9 +297,13 @@ static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         fprintf(stderr, "\n");
 #endif /* EC_DEBUG */
 
+#ifdef PILOT_LINK_0_12
+        ret = dlp_ReadAppBlock(dbi->pilot_socket, dbHandle, 0, 0xffff, pi_buf);
+        unpack_ExpenseAppInfo(tai, pi_buf->data, 0xffff);
+#else
         ret = dlp_ReadAppBlock(dbi->pilot_socket, dbHandle, 0, buffer, 0xffff);
-    
         unpack_ExpenseAppInfo(tai, buffer, 0xffff);
+#endif
 
 #ifdef EC_DEBUG
         fprintf(stderr, "Orig length %d, new length %d, orig data:\n", ret, i);
@@ -303,7 +326,8 @@ static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         }
         
         /* open one file for every category in Expense */
-        for(i = 0; i < 17; i++) {
+        for(i = 0; i < numhandles; i++) {
+                /* skip unused categories */
                 if(*tai->category.name[i] != '\0') {
                         LOG("Opening for cat %d: %s\n", i, category_path(i, c));
                         if((filehandle[i] = creat(category_path(i, c), GET_CONDUIT_CFG(c)->fileMode))== -1) {
@@ -321,7 +345,11 @@ static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
                 struct Expense t;
                 int attr, category, len;
 
+#ifdef PILOT_LINK_0_12
+                len = dlp_ReadRecordByIndex(dbi->pilot_socket, dbHandle, i, pi_buf, 0, &attr, &category);
+#else
                 len = dlp_ReadRecordByIndex(dbi->pilot_socket, dbHandle, i, buffer, 0, 0, &attr, &category);
+#endif
                 
                 /* at the end of all the records? */
                 if(len < 0)
@@ -330,7 +358,21 @@ static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
                 if((attr & dlpRecAttrDeleted) || (attr & dlpRecAttrArchived))
                         continue;
 
-                unpack_Expense(&t, buffer, len);
+#ifdef PILOT_LINK_0_12
+                unpack_Expense(&t, pi_buf->data, len);
+#else
+                unpack_Expense(&t, buffer, len);                
+#endif
+                /* PalmOS should give us a 4-bit category, and that's all that
+                 * pi-appinfo.h can cope with, but lets be cautious */
+                if (category < 0 || category >= 16) {
+                        g_warning ("Out-of-range category ID from device: %d\n", category);
+                        goto error;
+                }
+                if (filehandle[category] == -1) {
+                        g_warning ("Unexpected category ID from device: %d\n", category);
+                        goto error;
+                }
                 writeout_record(filehandle[category], &t, c);
                 free_Expense(&t);
         }
@@ -340,13 +382,19 @@ static gint copy_from_pilot( GnomePilotConduit *c, GnomePilotDBInfo *dbi )
         result = -1;
  exit:
         /* close all the opened filehandles */
-        for(i = 0; i < 17; i++)
+        for(i = 0; i < numhandles; i++)
                 if(filehandle[i] != -1)
                         close(filehandle[i]);
 
         /* Close the database */
         dlp_CloseDB(dbi->pilot_socket, dbHandle);
         
+#ifdef PILOT_LINK_0_12
+        if (pi_buf) {
+                pi_buffer_free (pi_buf);
+        }
+#endif
+         
         return( result );
 }
 
@@ -362,7 +410,7 @@ insert_ignore_space_cb (GtkEditable    *editable, const gchar    *text,
                         gint len, gint *position, void *data)
 {
         gint i;
-        gchar *curname;
+        const gchar *curname;
 
         curname = gtk_entry_get_text(GTK_ENTRY(editable));
         if (*curname == '\0' && len > 0) {
@@ -427,9 +475,9 @@ struct _FieldInfo
 };
 
 
-FieldInfo fields[] = { { N_("Expense Directory"), NULL, "ExpenseDir", insert_ignore_space_cb},
-                       { N_("Directory Mode"), NULL, "DirMode", insert_numeric_cb},
-                       { N_("File Mode"), NULL, "FileMode", insert_numeric_cb}, 
+static FieldInfo fields[] = { { N_("Expense Directory:"), NULL, "ExpenseDir", insert_ignore_space_cb},
+                       { N_("Directory Mode:"), NULL, "DirMode", insert_numeric_cb},
+                       { N_("File Mode:"), NULL, "FileMode", insert_numeric_cb}, 
                        { NULL, NULL, NULL, NULL}
 };
 
@@ -451,7 +499,8 @@ static GtkWidget
 	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
 
         /* set the date format */
-        label = gtk_label_new(_("Date Format"));
+        label = gtk_label_new(_("Date Format:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
         gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
 
         menu = GTK_MENU(gtk_menu_new());
@@ -470,7 +519,9 @@ static GtkWidget
         gtk_object_set_data(GTK_OBJECT(vbox), "DateFormat", optionMenu);
 
         /* set the writeout format */
-        label = gtk_label_new(_("Output Format"));
+        label = gtk_label_new(_("Output Format:"));
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+
         gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
 
         menu = GTK_MENU(gtk_menu_new());
@@ -499,8 +550,9 @@ static GtkWidget
         widget_offset = 3;
 	for(i = 0; i < count; i++) {
 		label = gtk_label_new(_(fields[i].name));
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, 
-                                 widget_offset+i, widget_offset+i+1, 0,0,0,0);
+                gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+                gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1,
+                	widget_offset+i, widget_offset+i+1);
 		if(fields[i].label_data!=NULL) {
 			gtk_object_set_data(GTK_OBJECT(vbox), fields[i].label_data, label);
 		}
